using System;
using System.Linq;
using System.Threading.Tasks;
using System.Collections.Generic;

using Microsoft.AspNetCore.Builder;
using Microsoft.AspNetCore.Hosting;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Hosting;
using Microsoft.Extensions.Logging;
using Microsoft.OpenApi.Models;

using Swashbuckle;
using Swashbuckle.AspNetCore.Swagger;
using Swashbuckle.AspNetCore.SwaggerGen;
using Swashbuckle.AspNetCore.SwaggerUI;

using SwaggerStudy.Services;
using Microsoft.OpenApi.Interfaces;

namespace SwaggerStudy
{
    /// <summary>
    /// 测试用
    /// </summary>
    public class StartupTest
    {
        public StartupTest(IConfiguration configuration)
        {
            Configuration = configuration;
        }

        public IConfiguration Configuration { get; }

        public void ConfigureServices(IServiceCollection services)
        {
            services
                .AddControllers()
                .AddJsonOptions(jsonOption=> 
                {
                    jsonOption.JsonSerializerOptions.Converters.Add(new DatetimeJsonConverter());
                    jsonOption.JsonSerializerOptions.Encoder= System.Text.Encodings.Web.JavaScriptEncoder.Create(System.Text.Unicode.UnicodeRanges.All);
                });

            //注入SwaggerGen
            services.AddSwaggerGen(setup =>
            {
                //注意：name参数即是DocumentName, 在SwaggerUi中有使用，配置的终结点里必须保持一致
                setup.SwaggerDoc(name: "DemoName", info: new OpenApiInfo()
                {
                    //标题(必须的)
                    Title = "Title:使用Swagger",

                    //文档版本(必须的)
                    Version = "大麦版",

                    //应用程序的简短描述
                    Description = "描述：Swagger学习项目，全面学习Swagger使用方法。",

                    //暴露的API的联系信息
                    Contact = new OpenApiContact() { Name = "天意工作室", Url = new Uri("https://www.wanggaofeng.cn"), Email = "bicijinlian@163.com" },

                    //公开的API的许可证信息
                    License = new OpenApiLicense() { Name = "木兰宽松许可证", Url = new Uri("http://license.coscl.org.cn/MulanPSL") },

                    //API服务条款的URL(必须采用URL格式)
                    TermsOfService = new Uri("http://license.coscl.org.cn/MulanPSL"),

                    //此对象可以使用规范扩展进行扩展
                    Extensions = new Dictionary<string, IOpenApiExtension>()
                    {
                    }
                });

                //包含xml注释
                //var xmlCommentFiles = System.IO.Directory.GetFiles(AppDomain.CurrentDomain.BaseDirectory, $"SwaggerStudy.*.xml", System.IO.SearchOption.TopDirectoryOnly);
                //foreach (var xmlFile in xmlCommentFiles)
                //{
                //    setup.IncludeXmlComments(xmlFile, true);
                //}
                
            });

            //配置SwaggerGen
            services.ConfigureSwaggerGen(setup =>
            {

            });

            services.AddTransient<StudentServer>();
        }

        public void Configure(IApplicationBuilder app, IWebHostEnvironment env)
        {
            if (env.IsDevelopment())
            {
                app.UseDeveloperExceptionPage();
            }

            app.UseSwagger(setup =>
            {
                setup.SerializeAsV2 = false;

                //使用请求上下文修改Swagger
                setup.PreSerializeFilters.Add((swagger, httpReq) =>
                {
                    //删除部分接口
                    swagger.Paths.Remove("/api/Student/GetAll");
                    swagger.Paths.Remove("/api/Student/GetAllAsync");

                    swagger.Servers = new List<OpenApiServer> { new OpenApiServer { Url = $"{httpReq.Scheme}://{httpReq.Host.Value}" } };
                });

                //更改Swagger JSON端点的路径:自定义路由必须包含{documentName}参数
                //默认为 /swagger/{documentName}/swagger.json
                //SwaggerUi里，保持一致
                //终结点并非SwaggerUI访问url
                setup.RouteTemplate = "/WebApi/swagger/{documentName}/swagger.json";

            });
            app.UseSwaggerUI(setup => 
            {
                //swagger 访问页面前缀：默认swagger
                //SwaggerUi首页地址：baseUrl+前辍+index.html 默认：http://localhost:8080/swagger/index.html
                setup.RoutePrefix = "Swagger";
                setup.SwaggerEndpoint("/WebApi/swagger/DemoName/swagger.json", "WebApi 测试版");
            });

            app.UseRouting();

            app.UseAuthorization();

            app.UseEndpoints(endpoints =>
            {
                endpoints.MapControllers();
            });
        }
    }
}
